@@ -1,14 +1,14 @@
 # -*- coding: utf-8 -*-
 """
-==========
 data_tools
 ==========
 """
 
-from typing import Union
+from typing import Union, List, Dict, Any, Optional
 import DateTime
 import datetime
 import vobject
+import pendulum
 from dateutil.parser import parse
 from itb_date_time import convert_datetime, str_to_date, str_to_DT, get_dateformat
 from itb_email_phone import process_phonenumber
@@ -19,7 +19,7 @@ time_chars = set('1234567890:')
 
 
 def any_2boolean(
-    value
+    value: Any
 ) -> bool:
     """ return a boolean for any value
 
@@ -34,14 +34,12 @@ def any_2boolean(
 
 
 def dict_from_dict_list(
-    dict_list: list,
-    key,
-    single_value: bool = None,
-    include_key: str = False
+    dict_list: List[dict],
+    key: str,
+    single_value: Optional[str] = None,
+    include_key: Optional[bool] = False
 ) -> dict:
-    """Create a dictionary from a list of dictionaries
-
-    .. note:: The annotated return type is not true if a single_value argument is given.
+    """ Create a dictionary from a list of dictionaries
 
     :param dictList: the list of dictionaries to be converted
     :param key: the key of the dictionaries to use as key for the new dictionary.
@@ -68,10 +66,10 @@ def dict_from_dict_list(
 
 
 def create_vcard(
-    data: dict,
+    data: Dict[str, str],
     returning: str = 'vcard'
 ) -> str:
-    """create a vCard from a dictionary
+    """ create a vCard from a dictionary
 
     .. note:: The serialized vcard has Windows line-breaks, which is fine, I guess
     .. note:: colons in notes are escaped in vobject
@@ -132,10 +130,10 @@ def create_vcard(
 
 
 def dict_2unicode(
-    d: dict,
+    d: Dict[Union[str, bytes], Any],
     encoding: str = 'utf-8'
 ) -> dict:
-    """ Converts all keys and values in a dictionary from bytes to unicode
+    """ Convert all keys and values in a dictionary from bytes to unicode
 
     All keys and values are changed from bytes to unicode, if applicable.
     Other data types are left unchaged, including other compound data types.
@@ -153,13 +151,33 @@ def dict_2unicode(
 
 def dict_2datatypes(
     d: dict,
-    metadata: dict = {},
+    metadata: Dict[str, str] = {},
     convert_keys: bool = False,
     convert_to_unicode: bool = False,
     dt: str = 'datetime',
-    fmt: str = '%d.%m.%Y'
-) -> dict:
-    """ convert all data of a dictionary to specific (json metadata) or guessed types """
+    fmt: str = None,
+    timezone: str = 'local'
+) -> Dict[Any, Any]:
+    """ Convert all data of a dictionary to specific (json metadata) or guessed types
+
+    :param d: dictionary
+    :param metadata: dictionary containing types for the keys:
+               - boolean
+               - int
+               - float
+               - date
+               - datetime: date and time
+               - time: conversion or not based on dt
+    :param convert_keys: convert keys to unicode
+    :param convert_to_unicode: convert all bytes to unicode in the process
+    :param dt: - datetime or dt
+               - DateTime or DT
+               - ignore or string
+               - pendulum
+               - default: datetime, because it's Python's default
+    :param fmt: format for datetime parsing
+    :param timezone: default is 'local', other options are 'UTC' or 'Europe/Vienna'
+    """
 
     if convert_keys:
         return {
@@ -174,7 +192,8 @@ def dict_2datatypes(
                 key=key,
                 convert_to_unicode=convert_to_unicode,
                 dt=dt,
-                fmt=fmt
+                fmt=fmt,
+                timezone=timezone
             )
             for key, val in d.items()
         }  # dict comprehension
@@ -186,27 +205,51 @@ def dict_2datatypes(
                 key=key,
                 convert_to_unicode=convert_to_unicode,
                 dt=dt,
-                fmt=fmt
+                fmt=fmt,
+                timezone=timezone
             )
             for key, val in d.items()
         }  # dict comprehension
 
 
 def dictlist_2datatypes(
-    dictlist: str,
-    metadata: dict = {},
+    dictlist: List[Dict[Any, Any]],
+    metadata: Dict[Any, Any] = {},
     convert_keys: bool = False,
     convert_to_unicode: bool = False,
     dt: str = 'datetime',
-    fmt: str = '%d.%m.%Y'
+    fmt: str = None,
+    timezone: str = 'local'
 ) -> list:
+    """ Convert all data of a list of dictionaries to specific (json metadata) or guessed types
+
+    :param dictList: list of dictionaries
+    :param metadata: dictionary containing types for the keys:
+                - boolean
+                - int
+                - float
+                - date
+                - datetime: date and time
+                - time: conversion or not based on dt
+    :param convert_keys: convert keys to unicode
+    :param convert_to_unicode: convert all bytes to unicode in the process
+    :param dt: - datetime or dt
+               - DateTime or DT
+               - ignore or string
+               - pendulum
+               - default: datetime, because it's Python's default
+    :param fmt: format for datetime parsing
+    :param timezone: default is 'local', other options are 'UTC' or 'Europe/Vienna'
+    """
     return [
         dict_2datatypes(
             d=dic,
             metadata=metadata,
             convert_keys=convert_keys,
             convert_to_unicode=convert_to_unicode,
-            dt=dt
+            dt=dt,
+            fmt=fmt,
+            timezone=timezone
         )
         for dic
         in dictlist
@@ -214,8 +257,8 @@ def dictlist_2datatypes(
 
 
 def sort_dictlist(
-    dictlist: list,
-    keys: (str, list),
+    dictlist: List[dict],
+    keys: Union[str, List[str]],
     reverse: bool = False
 ) -> list:
     """ Sorts a list of dictionaries
@@ -259,17 +302,18 @@ def value_2datatype(
     convert_to_unicode: bool = False,
     encoding: str = 'utf-8',
     dt: str = 'datetime',
-    fmt: str = '%d.%m.%Y',  # only for datetime
-    UTC: bool = False
+    fmt: str = None,
+    timezone: str = 'local'
 ) -> Union[str, int, float]:
     """ Convert a value to a datatype
 
     this function has two modes:
 
-    1. it is provided a type for the conversion
+    1. a type is provided for the conversion
     2. it makes educated guesses in converting a string
 
     :param value:
+    :param typ: type for the conversion
     :param metadata: dictionary containing types for the keys:
                - boolean
                - int
@@ -282,7 +326,10 @@ def value_2datatype(
     :param encoding: if bytes are present, use this encoding to convert them to unicode
     :param dt: - datetime or dt
                - DateTime or DT
+               - ignore or string
+               - pendulum
                - default: datetime, because it's Python's default
+    :param timezone: default is 'local', other options are 'UTC' or 'Europe/Vienna'
     """
     if typ:
         if typ == 'boolean':
@@ -312,6 +359,17 @@ def value_2datatype(
                     value,
                     datefmt='international'
                 ) + 0.5  # to avoid day-shifting due to timezones, use 12am
+            elif dt == 'pendulum':
+                if fmt:
+                    p_dt = pendulum.from_format(value, fmt)
+                else:
+                    p_dt = pendulum.parse(value)
+
+                if p_dt.tzinfo == 'UTC' and timezone == 'local':
+                    return p_dt.in_tz(pendulum.local_timezone())
+                elif timezone:
+                    return p_dt.in_tz(timezone)
+                return p_dt.date()
             elif dt in ('ignore', 'string'):
                 return value
             else:
@@ -328,6 +386,17 @@ def value_2datatype(
                     )
                 except DateTime.interfaces.SyntaxError:
                     return value
+            elif dt == 'pendulum':
+                if fmt:
+                    p_dt = pendulum.from_format(value, fmt)
+                else:
+                    p_dt = pendulum.parse(value)
+
+                if p_dt.tzinfo == 'UTC' and timezone == 'local':
+                    return p_dt.in_tz(pendulum.local_timezone())
+                elif timezone:
+                    return p_dt.in_tz(timezone)
+                return p_dt
             elif dt in ('ignore', 'string'):
                 return value
             else:
@@ -340,7 +409,6 @@ def value_2datatype(
                 return value  # DT: return the string
 
     else:
-
         if isinstance(value, (bool, int, float)):
             return value
 
@@ -374,12 +442,7 @@ def value_2datatype(
                     datestring=value,
                     checkonly=True
                 ):
-                    if ':' in value:
-                        return convert_datetime(
-                            date=value,
-                            convert_to=dt
-                        )
-                    elif dt in ('datetime', 'dt'):
+                    if dt in ('datetime', 'dt'):
                         return str_to_date(
                             datestring=value
                         )
@@ -387,6 +450,22 @@ def value_2datatype(
                         return str_to_DT(
                             datestring=value
                         )
+                    elif dt == 'pendulum':
+                        if fmt:
+                            p_dt = pendulum.from_format(value, fmt)
+                        else:
+                            p_dt = pendulum.parse(value)
+                
+                        if p_dt.tzinfo == 'UTC' and timezone == 'local':
+                            return p_dt.in_tz(pendulum.local_timezone())
+                        elif timezone:
+                            return p_dt.in_tz(timezone)
+                        return p_dt
+                    else:
+                        return convert_datetime(
+                            date=value,
+                            convert_to=dt
+                        )
 
             except Exception:
                 # in case it's a subset of datetime_chars but not a date or datetime or time
