@@ -1,16 +1,26 @@
 # coding=utf-8
 """
-===============
 date_time_tools
 ===============
 """
-
+from typing import Union
 import datetime
 import DateTime
+import pendulum
 from dateutil.parser import parse
 from dateutil.parser._parser import ParserError
 
 
+TIME_ELS = ('h', 'm', 's', 'ms')
+
+time_dividers = {
+    'h': 1/24,
+    'm': 1/24/60,
+    's': 1/24/60/60,
+    'ms': 1/24/60/60/1000
+}
+
+
 def is_valid_datetime_string(
     datestring: str,
     returning: str = None
@@ -27,6 +37,8 @@ def is_valid_datetime_string(
     a conflict with 4-digit year values, it is considered invalid
 
     :param returning: can return a tuple
+
+    .. note:: This does not work with ISO8601 strings! (returns False)
     """
     return str_to_dt(
         datestring=datestring,
@@ -47,6 +59,14 @@ def get_dateformat(
     :todo: times!
     :raises ValueError: if an invalid datetime or DateTime string is provided
     """
+    # first let's check for ISO8601 strings
+    if checkonly:
+        try:
+            pendulum.parse(datestring)
+            return True
+        except pendulum.parsing.exceptions.ParserError:
+            pass  # don't return False, because it may be a valid date string
+
     timefmt = ''
     if ' ' in datestring:
         try:
@@ -113,7 +133,7 @@ def get_dateformat(
 
 def str_to_dt(
     datestring: str,
-    fmt: (str, None) = None,
+    fmt: str = None,
     checkonly: bool = False,
     returning: str = None
 ) -> datetime.datetime:
@@ -181,7 +201,7 @@ def str_to_dt(
 
 def str_to_DT(
     datestring: str,
-    fmt: (str, None) = None
+    fmt: str = None
 ) -> DateTime.DateTime:
     """ Convert a string to Datetime.Datetime
 
@@ -203,9 +223,24 @@ def str_to_DT(
         )
 
 
+def str_to_pendulum(
+    datestring: str,
+    fmt: str = None
+) -> pendulum.DateTime:
+    """ Convert a string to pendulum.DateTime
+
+    :param fmt: default: %d.%m.%Y (if not specified, the datestring is assumed to be valid ISO8601)
+    """
+
+    if not fmt:
+        return pendulum.parse(datestring)
+    else:
+        return pendulum.from_format(datestring, fmt)
+
+
 def str_to_date(
     datestring: str,
-    fmt: (str, None) = None
+    fmt: str = None
 ) -> datetime.date:
     """ Convert a string to datetime.date
 
@@ -246,6 +281,38 @@ def DT_to_date(
     )
 
 
+def DT_to_pendulum(
+    DT: DateTime.DateTime
+) -> pendulum.DateTime:
+    """ Convert DateTime.DateTime to pendulum.DateTime """
+
+    tz = DT.timezone()
+
+    offset = None
+
+    if '+' in tz:
+        offset = int(tz.split('+')[-1])
+    elif '-' in tz:
+        offset = - int(tz.split('-')[-1])
+    
+    if offset is not None:
+        if not offset:
+            tz = 'UTC'
+        else:
+            tz = pendulum.FixedTimezone(offset * 3600)
+    else:
+        tz = pendulum.timezone(tz)
+
+    return pendulum.datetime(
+        DT.year(),
+        DT.month(),
+        DT.day(),
+        DT.hour(),
+        DT.minute(),
+        tz = tz
+    )
+
+
 def date_to_dt(
     date: datetime.date,
     H=None, M=None
@@ -280,16 +347,46 @@ def date_to_DT(
     return DateTime.DateTime(date_to_dt(date))
 
 
+def dt_to_pendulum(
+    dt: datetime.datetime
+) -> pendulum.DateTime:
+    """ Convert datetime.datetime to pendulum.DateTime """
+
+    return pendulum.datetime(
+        dt.year,
+        dt.month,
+        dt.day,
+        dt.hour,
+        dt.minute
+    
+    )
+
+
+def pendulum_to_DT(
+    pendulum_dt: pendulum.DateTime
+) -> DateTime.DateTime:
+    """ Convert pendulum.DateTime to DateTime.DateTime """
+
+    return DateTime.DateTime(
+        pendulum_dt.year,
+        pendulum_dt.month,
+        pendulum_dt.day,
+        pendulum_dt.hour,
+        pendulum_dt.minute
+    ).toZone(pendulum_dt.timezone.name)
+
+
 def convert_datetime(
-    date: (
+    date: Union[
         str,
         datetime.datetime,
         datetime.date,
-        DateTime.DateTime
-    ),
+        DateTime.DateTime,
+        pendulum.DateTime
+    ],
     convert_to: str,
     fmt: str = None
-) -> (datetime.datetime, datetime.date, DateTime.DateTime, str):
+) -> Union[datetime.datetime, datetime.date, DateTime.DateTime, str]:
     """ Conversion of date and time formats
 
     :param convertTo:
@@ -311,6 +408,17 @@ def convert_datetime(
             return DT_to_dt(date)
         elif convert_to in ('str', 'string'):
             return date.strftime(fmt)
+        elif convert_to == 'pendulum':
+            return DT_to_pendulum(date)
+    elif isinstance(date, pendulum.DateTime):
+        if convert_to == 'datetime':
+            return date
+        elif convert_to == 'date':
+            return date.date()
+        elif convert_to == 'DateTime':
+            return pendulum_to_DT(date)
+        elif convert_to in ('str', 'string'):
+            return date.strftime(fmt)
     elif isinstance(date, str):
         if convert_to in ('str', 'string'):
             if str_to_date(date, checkonly=True, fmt=fmt):
@@ -323,6 +431,8 @@ def convert_datetime(
             return str_to_dt(date, fmt=fmt)
         elif convert_to == 'DateTime':
             return str_to_DT(date, fmt=fmt)
+        elif convert_to == 'pendulum':
+            return pendulum.parse(date)
     elif isinstance(date, datetime.datetime):
         # datetime.datetime has to be checked BEFORE
         # datetime.date (isinstance(datetime.datetime, datetime.date) == True !)
@@ -334,6 +444,8 @@ def convert_datetime(
             return DateTime.DateTime(date)
         elif convert_to in ('str', 'string'):
             return date.strftime(fmt)
+        elif convert_to == 'pendulum':
+            return dt_to_pendulum(date)
     elif type(date) is datetime.date:  # isinstance does not work
         if convert_to == 'date':
             return date
@@ -343,15 +455,21 @@ def convert_datetime(
             return date_to_DT(date)
         elif convert_to in ('str', 'string'):
             return date.strftime(fmt)
+        elif convert_to == 'pendulum':
+            return pendulum.datetime(
+                date.year,
+                date.month,
+                date.day
+            )
 
 
 def get_calendar_week(
-    date: (
+    date: Union[
         str,
         datetime.datetime,
         datetime.date,
         DateTime.DateTime
-    )
+    ]
 ) -> int:
     """ Get calendard week (week nb of year) from a date
 
@@ -364,12 +482,12 @@ def get_calendar_week(
 
 
 def get_dow_number(
-    date: (
+    date: Union[
         str,
         datetime.datetime,
         datetime.date,
         DateTime.DateTime
-    )
+    ]
 ) -> int:
     """ Get day of week number from a date
 
@@ -382,12 +500,12 @@ def get_dow_number(
 
 
 def get_isocalendar(
-    date: (
+    date: Union[
         str,
         datetime.datetime,
         datetime.date,
         DateTime.DateTime
-    )
+    ]
 ) -> datetime.date:
     """ Returns the isocalendar tuple: (year, woy, downb)
 
@@ -441,6 +559,7 @@ def daterange_from_week(
            - DateTime
            - datetime (default)
            - date
+           - pendulum
     :param fmt: default: '%d.%m.%Y'
 
     .. note:: Author: Andreas Bruhn, https://groups.google.com/forum/#!topic/de.comp.lang.python/p8LfbNMIJ5c
@@ -478,6 +597,7 @@ def dates_from_week(
         - DateTime (default)
         - datetime
         - date
+        - pendulum
 
     :param fmt: default: '%d.%m.%Y'
     :param endofweek: 1 to 7 (monday to sunday)
@@ -515,7 +635,7 @@ def day_from_week(
     weekday: int = 1,
     returning: str = 'date',
     fmt: str = '%d.%m.%Y'
-) -> (datetime.datetime, datetime.date, DateTime.DateTime):
+) -> Union[datetime.datetime, datetime.date, DateTime.DateTime, pendulum.DateTime]:
     """Get day from a week
 
     :param weekday: begins with 1 = monday
@@ -524,6 +644,7 @@ def day_from_week(
        - DateTime
        - datetime (default)
        - date
+       - pendulum
 
     :param fmt: default: '%d.%m.%Y'
     """
@@ -541,7 +662,7 @@ def monday_from_week(
     week: int,
     returning: str = 'date',
     fmt='%d.%m.%Y'
-) -> (datetime.datetime, datetime.date, DateTime.DateTime):
+) -> Union[datetime.datetime, datetime.date, DateTime.DateTime, pendulum.DateTime]:
     """Get monday from a week
 
     :param: weekday begins with 1 = monday
@@ -572,7 +693,7 @@ def last_week_of_year(
 
 
 def remove_time(
-    date_time: (datetime.datetime, DateTime.DateTime),
+    date_time: Union[datetime.datetime, DateTime.DateTime, pendulum.DateTime],
     returning: str = None
 ):
     """Remove time from a datetime or DateTime object
@@ -617,3 +738,157 @@ def remove_time(
                 date_time.month(),
                 date_time.day()
             )
+    elif isinstance(date_time, pendulum.DateTime):
+        if not returning or returning == 'pendulum':
+            return date_time.start_of('day')
+        elif returning == 'date':
+            return date_time.date()
+        elif returning == 'datetime':
+            return date_time.datetime()
+        elif returning == 'DateTime':
+            return DateTime.DateTime(
+                date_time.year,
+                date_time.month,
+                date_time.day
+            )
+
+
+def create_timedelta(
+    timestr: str,
+    dt: str
+):
+    """create a timedelta or a number from a time string
+
+    :param timestr: format or H:M or H:M:S or H:M:S:ms, possibly with day: 4T03:12:31
+    :param returningdt: dt, DT or pendulum
+    """
+    days_time = timestr.split('T')
+    timestr = days_time[-1]
+    days = 0
+    if len(days_time) > 1:
+        days = days_time[0]
+
+    time_list = timestr.split(':')
+    time_elements = {}
+
+    for i, el in enumerate(TIME_ELS):
+        try:
+            time_elements[el] = int(time_list[i])
+        except IndexError:
+            time_elements[el] = 0
+
+    if dt == 'datetime':
+        t_delta = datetime.timedelta(
+            days=days,
+            hours=time_elements['h'],
+            minutes=time_elements['m'],
+            seconds=time_elements['s'],
+            milliseconds=time_elements['ms'],
+        )
+    elif dt == 'DateTime':
+        t_delta = days
+        for el in TIME_ELS:
+            t_delta += time_elements[el] * time_dividers[el]
+    elif dt == 'pendulum':
+        t_delta = pendulum.duration(
+            days=days,
+            hours=time_elements['h'],
+            minutes=time_elements['m'],
+            seconds=time_elements['s'],
+            milliseconds=time_elements['ms'],
+        )
+
+    return t_delta
+
+
+def add_time(
+    date_time: Union[datetime.datetime, DateTime.DateTime, pendulum.DateTime],
+    timestr: str,
+    subtract: bool = False,
+    returning: str = None
+):
+    """add or remove time from a datetime or DateTime object
+
+    :param date_time: must be a datetime or DateTime object
+    :param returning: if None, the same type as date_time is returned,
+            otherwise valid values are "datetime" and "DateTime"
+
+    We can safely assume that that the time string is in the format
+    H:M or H:M:S or H:M:S:ms
+    or
+    dTH:M etc... (ex.: 4T03:12:31)
+    """
+    if subtract or timestr.startswith('-'):
+        subtract = True
+        timestr = timestr[1:]
+
+    if isinstance(date_time, datetime.datetime):
+        dt = 'datetime'
+    elif isinstance(date_time, datetime.date):
+        date_time = date_to_dt(date_time)
+        dt = 'datetime'
+    elif isinstance(date_time, DateTime.DateTime):
+        dt = 'DateTime'
+    elif isinstance(date_time, pendulum.DateTime):
+        dt = 'pendulum'
+    else:
+        raise Exception('date_time has to be datetime.datetime, datetime.date or DateTime.DateTime!')
+
+    t_delta = create_timedelta(
+        timestr=timestr,
+        dt=dt
+    )
+
+    if subtract:
+        return date_time - t_delta
+    else:
+        return date_time + t_delta
+
+
+def unravel_duration(
+    duration: str,
+    returning: str = 'list'
+):
+    """split a duration string into components
+
+    duration comes in the format nTH:M:S:ms
+    (days, hours, minutes, seconds, milliseconds)
+
+    @param returning: list, tuple oder dictionary
+    """
+    if 'T' in duration:
+        days, rest = duration.split('T')
+        days = int(days)
+    else:
+        days = 0
+        rest = duration
+
+    elements = rest.split(':')
+
+    hours = minutes = seconds = milliseconds = 0
+    time_elements = [hours, minutes, seconds, milliseconds]
+
+    for i, (element, time_measure) in enumerate(zip(elements, time_elements.copy())):
+        time_elements[i] = int(element)
+
+    time_elements.insert(0, days)
+
+    for i, divider in zip(
+        range(4, -1, -1),
+        (1000, 60, 60, 24)
+    ):
+        time_element = int(time_elements[i] / divider)
+        time_elements[i] -= time_element * divider
+        time_elements[i - 1] += time_element
+
+    if returning == 'list':
+        return time_elements
+    elif returning in ('dict', 'dictionary'):
+        d = {}
+        for name, val in zip(('days', 'hours', 'minutes', 'seconds', 'milliseconds'), time_elements):
+            d[name] = val
+        return d
+    elif returning == 'tuple':
+        tuple(time_elements)
+    else:
+        raise Exception('argument returning must be list, dictionary or tuple.')
