@@ -5,11 +5,12 @@ email_phone_tools
 =================
 """
 
-from typing import Union
+from typing import Union, List, Dict
 import phonenumbers
 from email.mime.multipart import MIMEMultipart
 from email.mime.text import MIMEText
 from email import encoders
+from email.header import Header
 from email.mime.base import MIMEBase
 
 from itb_locales import get_locale
@@ -69,7 +70,7 @@ def create_email_message(
     cc: str = None,
     bcc: str = None,
     html: str = None,
-    attachments: list = None,
+    attachments: List[Dict] = None,
     encoding: str = 'utf-8'
 ) -> str:
     """ Create a valid email message
@@ -80,10 +81,13 @@ def create_email_message(
 
     if attachments:
         msg = MIMEMultipart('mixed')
+        body = MIMEMultipart('alternative')
     elif text and html:
         msg = MIMEMultipart('alternative')
+        body = None
     else:
         msg = MIMEMultipart('alternative')
+        body = None
 
     msg['Subject'] = subject
     msg['From'] = mail_from
@@ -100,11 +104,21 @@ def create_email_message(
     if bcc:
         msg['BCC'] = bcc
 
-    parts = []
-    parts.append(MIMEText(text, 'plain', encoding))
-
+    mime_text = MIMEText(text, 'plain', encoding)
+    mime_html = None
+    
     if html:
-        parts.append(MIMEText(html, 'html', encoding))
+        mime_html = MIMEText(html, 'html', encoding)
+
+    if body:
+        body.attach(mime_text)
+        if html:
+            body.attach(mime_html)
+        msg.attach(body)
+    else:
+        msg.attach(mime_text)
+        if html:
+            msg.attach(mime_html)
 
     for attachment in attachments or []:
         attPart = MIMEBase(
@@ -117,10 +131,7 @@ def create_email_message(
             'Content-Disposition',
             f"attachment; filename={attachment['filename']}"
         )
-        parts.append(attPart)
-
-    for part in parts:
-        msg.attach(part)
+        msg.attach(attPart)
 
     return msg
 
