@@ -1,6 +1,5 @@
 # -*- coding: utf-8 -*-
 """
-=======================
 restricted_python_tools
 =======================
 
@@ -41,7 +40,7 @@ def contains_any(
 def remove_duplicates(lst: list) -> list:
     """ Removes duplicates from a list
 
-    .. note:: restricted Python does not allow sets
+    .. note:: restricted Python does not allow sets. Also be aware that the order might be changed.
     """
 
     return list(set(lst))
