@@ -1,9 +1,9 @@
 # -*- coding: utf-8 -*-
 """
-===============
 www_tools
-===============
+=========
 """
+from typing import Union, List, Dict
 import urllib.parse
 
 zope_data_types = {
@@ -27,8 +27,8 @@ def change_query_string(
     url: str = None,
     query_string: str = None,
     delete_remaining: bool = False,
-    returning='auto'
-):
+    returning: str = 'auto'
+) -> str:
     """
     Change arguments in a query string
 
