@@ -2,35 +2,43 @@
 """
 run the test from the sr/invesytoolbox directory:
 python ../tests/test_date_time.py
+
+TODO test for: 
+- pendulum
+- checking timezone handling
 """
 
 import sys
 import unittest
 import datetime
 import DateTime
+from random import randint
 
 sys.path.append(".")
 
-from itb_date_time import \
-    convert_datetime, \
-    date_to_dt, \
-    date_to_DT, \
-    DT_to_dt, \
-    DT_to_date, \
-    daterange_from_week, \
-    dates_from_week, \
-    day_from_week, \
-    get_calendar_week, \
-    get_dateformat, \
-    get_dow_number, \
-    get_isocalendar, \
-    is_valid_datetime_string, \
-    last_week_of_year, \
-    monday_from_week, \
-    remove_time, \
-    str_to_dt, \
-    str_to_date, \
-    str_to_DT
+from itb_date_time import (
+    add_time,
+    convert_datetime,
+    date_to_dt,
+    date_to_DT,
+    DT_to_dt,
+    DT_to_date,
+    daterange_from_week,
+    dates_from_week,
+    day_from_week,
+    get_calendar_week,
+    get_dateformat,
+    get_dow_number,
+    get_isocalendar,
+    is_valid_datetime_string,
+    last_week_of_year,
+    monday_from_week,
+    remove_time,
+    str_to_dt,
+    str_to_date,
+    str_to_DT,
+    unravel_duration
+)
 
 get_dateformat_error_str_part = ' is not a valid date or datetime string.'
 
@@ -179,6 +187,39 @@ dates_for_checking = {
     }
 }
 
+durations = {
+    '5T23:04:17': {
+        'list': [5, 23, 4, 17, 0],
+        'dictionary': {
+            'days': 5,
+            'hours': 23,
+            'minutes': 4,
+            'seconds': 17,
+            'milliseconds': 0
+        }
+    },
+    '45:3:18:1500': {
+        'list': [1, 21, 3, 19, 500],
+        'dictionary': {
+            'days': 1,
+            'hours': 21,
+            'minutes': 3,
+            'seconds': 19,
+            'milliseconds': 500
+        }
+    },
+    '4T58:68:17': {
+        'list': [6, 11, 8, 17, 0],
+        'dictionary': {
+            'days': 6,
+            'hours': 11,
+            'minutes': 8,
+            'seconds': 17,
+            'milliseconds': 0
+        }
+    }
+}
+
 
 class Test_date_time(unittest.TestCase):
 
@@ -518,6 +559,54 @@ class Test_date_time(unittest.TestCase):
                     v.get(f'{dt}_notime')
                 )
 
+    def test_add_time(self):
+        for _ in range(5):
+            date_time = [
+                randint(1920, 2023),
+                randint(1, 12),
+                randint(1, 28),
+                randint(1, 11),
+                randint(1, 29),
+                randint(1, 29)
+            ]
+            add_time_list = [
+                randint(1, 11),
+                randint(1, 29),
+                randint(1, 29)
+            ]
+
+            add_time_str = ':'.join([str(n).rjust(2, '0') for n in add_time_list])
+
+            added_time = [
+                *date_time[:3],
+                date_time[3] + add_time_list[0],
+                date_time[4] + add_time_list[1],
+                date_time[5] + add_time_list[2]
+            ]
+
+            self.assertEqual(
+                datetime.datetime(*added_time),
+                add_time(
+                    datetime.datetime(*date_time),
+                    add_time_str
+                )
+            )
+            self.assertEqual(
+                DateTime.DateTime(*added_time),
+                add_time(
+                    DateTime.DateTime(*date_time),
+                    add_time_str
+                )
+            )
+    def test_unravel_duration(self):
+        # to be implemented
+        for s, d in durations.items():
+            for returning in ('list', 'dictionary'):
+                self.assertEqual(
+                    unravel_duration(duration=s, returning=returning),
+                    d.get(returning)
+                )
+
 
 if __name__ == '__main__':
     unittest.main()
